# -*- coding: utf-8 -*-
from enum import Enum


class ContentType(Enum):
    ILLUST = "illust"
    MANGA = "manga"
    NOVEL = "novel"
    USER = "user"


class Restrict(Enum):
    PUBLIC = "public"
    PRIVATE = "private"
    ALL = "all"


class SearchAspectRatio(Enum):
    PORTRAIT = "portrait"
    LANDSCAPE = "landscape"
    SQUARE = "square"


class SearchIllustTool(Enum):
    PHOTOSHOP = "photoshop"
    ILLUSTRATOR = "illustrator"
    FIREWORKS = "fireworks"


class SearchSize(Enum):
    MINIMUM = "minimum"
    MEDIUM = "medium"
    LARGE = "large"


class SearchSort(Enum):
    DESC = "date_desc"
    ASC = "date_asc"
    POPULAR_DESC = "popular_desc"


class XRestrict(Enum):
    GENERAL = 0
    R18 = 1
    R18G = 2



# -*- coding: utf-8 -*-
import requests

from ..exceptions import PyxivError, PyxivAuthFailed
from ..constants import (
    OS, OS_VERSION, VERSION, USER_AGENT, CLIENT_ID, CLIENT_SECRET, OAUTH_URL
)


class OAuth(object):

    def __init__(self, session=None):
        super(OAuth, self).__init__()
        self._session = requests.Session() if session is None else session
        self._session.headers.update({
            'User-Agent': USER_AGENT,
            'App-OS': OS,
            'App-OS-Version': OS_VERSION,
            'App-Version': VERSION,
        })

        self._access_token = None
        self._refresh_token = None

        self._id = None
        self._name = None
        self._username = None
        self._device_token = 'pixiv'

        self.__password = None

    def _req(self, method, url, params=None, data=None, headers=None,
             is_retry=False):
        req = self._session.request(method, url, params, data, headers)

        if req.status_code == 400:
            try:
                r = req.json()
                if r['errors']['system']['code'] == 1508:
                    if is_retry:
                        raise PyxivError(req.text)
                    self._auth()
                    self._req(method, url, params, data, headers, True)
            except (ValueError, KeyError):
                req.raise_for_status()
        elif req.status_code != 200:
            raise PyxivError(req.text)
        else:
            return req.json()

    def _auth(self, username=None, password=None):
        data = {
            'client_id': CLIENT_ID,
            'client_secret': CLIENT_SECRET,
            'device_token': self._device_token,
            'get_secure_url': True,
        }

        if username is not None and password is not None:
            self._username = username
            self.__password = password
            data['grant_type'] = 'password'
            data['username'] = username
            data['password'] = password
        else:
            data['grant_type'] = 'refresh_token'

        self._session.headers.pop('Authorization', None)
        url = OAUTH_URL + "/auth/token"
        req = self._session.post(url, data=data)

        if req.status_code != 200:
            try:
                r = req.json()
                if r['errors']['system']['code'] == 1508:
                    raise PyxivAuthFailed(req.text)
            except KeyError:
                raise PyxivAuthFailed(req.text)

        r = req.json()['response']
        self._access_token = r['access_token']
        self._refresh_token = r['refresh_token']
        self._id = r['user']['id']
        self._name = r['user']['name']
        self._username = r['user']['account']
        self._device_token = r['device_token']

        self._session.headers.update({
            'Authorization': 'Bearer ' + self._access_token
        })

    def _login(self, username, password, device_token=None):
        if device_token is not None:
            self.device_token = device_token
        self._auth(username, password)



ACCOUNTS_URL = "https://accounts.pixiv.net"
APP_URL = "https://app-api.pixiv.net"
OAUTH_URL = "https://oauth.secure.pixiv.net"
SKETCH_URL = "https://sketch.pixiv.net"

OS = "android"
OS_VERSION = "7.0"
OS_MODEL = "SM-A310F"
VERSION = "5.0.96"

USER_AGENT = f"PixivAndroidApp/{VERSION} (Android {OS_VERSION}; {OS_MODEL})"

CLIENT_ID = "MOBrBDS8blbauoSck0ZfDbtuzpyT"
CLIENT_SECRET = "lsACyCD94FhDUtGTXi3QzcFE2uU1hqtDaKeqrdwj"
PROVISIONAL_TOKEN = "l-f9qZ0ZyqSwRyZs8-MymbtWBbSxmCu1pmbOlyisou8"



# -*- coding: utf-8 -*-


class PyxivError(Exception):
    """Pixiv Base Exception"""


class PyxivAuthFailed(PyxivError):
    """Authentication Error"""


class PyxivNotAuthenticatedError(PyxivError):
    """Not Authenticated Error"""



# -*- coding: utf-8 -*-
from .api import Account, App, OAuth, Sketch


class Pyxiv(object):
    """Pixiv private API wrapper."""

    def __init__(self, session=None):
        super(Pyxiv, self).__init__()
        self._oauth = OAuth(session)
        self.account = Account(self._oauth)
        self.app = App(self._oauth)
        self.sketch = Sketch(self._oauth)

    def __getattr__(self, name):
        for api in {self.account, self.app, self.sketch}:
            try:
                return getattr(api, name)
            except AttributeError:
                pass
        raise AttributeError(f"{__name__} object has no attribute {name}")

    def login(self, username, password, provisional=False, device_token=None):
        if provisional:
            account = self.account.createProvisionalAccount(
                'Pyxiv', 'pixiv_android_app_provisional_account')
            self._oauth._login(account)
            return

        self._oauth._login(username, password, device_token)



# -*- coding: utf-8 -*-
from .base import PixivApiBase
from ..constants import ACCOUNTS_URL, PROVISIONAL_TOKEN
from ..decorators import POST, Field, Authenticated


class Account(PixivApiBase):

    PROVISIONAL_AUTH = {'Authorization': 'Bearer ' + PROVISIONAL_TOKEN}

    def __init__(self, oauth):
        super(Account, self).__init__(oauth, ACCOUNTS_URL)

    @POST("/api/provisional-accounts/create", headers=PROVISIONAL_AUTH)
    def createProvisionalAccount(self, user_name: Field, ref: Field):
        """Create a temporary account."""

    @Authenticated
    @POST("/api/account/edit")
    def editAccount(self, new_mail_address, new_user_account, current_password,
                    new_password):
        pass



# -*- coding: utf-8 -*-
from .base import PixivApiBase
from ..constants import SKETCH_URL
from ..decorators import GET, POST, Field, Path, Authenticated


class Sketch(PixivApiBase):

    def __init__(self, oauth):
        super(Sketch, self).__init__(oauth, SKETCH_URL)

    @GET("/api/lives/{live_uid}.json")
    def getLive(self, live_uid: Path):
        pass

    @GET("/api/lives/{live_uid}/chats.json")
    def getLiveChats(self, live_uid: Path):
        pass

    @Authenticated
    @POST("/api/lives/{live_uid}/hearts.json")
    def postHeart(self, live_uid: Path, count: Field, is_first: Field):
        pass

    @Authenticated
    @POST("/api/lives/{live_uid}/chats.json")
    def postLiveChats(self, live_uid: Path, message: Field):
        pass

    @Authenticated
    @POST("/api/lives/{live_uid}/members.json")
    def postLiveMembers(self, live_uid: Path):
        pass



# -*- coding: utf-8 -*-


class PixivApiBase(object):

    def __init__(self, oauth, base_url):
        super(PixivApiBase, self).__init__()
        self._oauth = oauth
        self._base_url = base_url



# -*- coding: utf-8 -*-
from functools import partial as functools_partial, wraps
from inspect import getfullargspec

from uritemplate import URITemplate
from decorator import decorate

from .exceptions import PyxivNotAuthenticatedError


class Field:
    """Request body field"""


class Path:
    """URITemplate parameter"""


class Query:
    """URL query parameter"""


def Authenticated(func):
    def wrapper(f, *args, **kwargs):
        if 'Authorization' not in args[0]._oauth._session.headers:
            raise PyxivNotAuthenticatedError('You are not logged in!')
        return func(*args, **kwargs)
    return decorate(func, wrapper)


def Request(method, path, returns=None, **option):
    def wrapper(func):
        @wraps(func)
        def inner_wrapper(f, *args, **kwargs):
            argspec = getfullargspec(func)
            uri = URITemplate(path)
            field, path_, query = {}, {}, {}

            # Parse each args and put them in the right place
            for n, (key, val) in enumerate(argspec.annotations.items(), 1):
                if val is Field:
                    field[key] = args[n]
                elif val is Path:
                    path_[key] = args[n]
                elif val is Query:
                    query[key] = args[n]

            option['data'] = field
            option['params'] = query

            url = args[0]._base_url + str(uri.partial(path_))
            result = args[0]._oauth._req(method, url, **option)
            if returns is None:
                return result
            return returns(result)
        return decorate(func, inner_wrapper)
    return wrapper


GET = functools_partial(Request, 'GET')
POST = functools_partial(Request, 'POST')



# -*- coding: utf-8 -*-
from .base import PixivApiBase
from ..constants import APP_URL
from ..decorators import GET, POST, Field, Query, Authenticated


class App(PixivApiBase):

    def __init__(self, oauth):
        super(App, self).__init__(oauth, APP_URL)

    @Authenticated
    @POST("/v1/illust/comment/delete")
    def deleteIllustComment(self, comment_id: Field):
        pass

    @Authenticated
    @POST("/v1/novel/comment/delete")
    def deleteNovelComment(self, comment_id: Field):
        pass

    @GET("/v1/application-info/android")
    def getApplicationInfo(self):
        pass

    @GET("/v1/emoji")
    def getEmoji(self):
        pass

    @Authenticated
    @GET("/v2/illust/follow")
    def getFollowIllusts(self, restrict: Query):
        pass

    @Authenticated
    @GET("/v1/novel/follow")
    def getFollowNovels(self, restrict: Query):
        pass

    @Authenticated
    @GET("/v1/user/follow/detail")
    def getFollowUserDetail(self, user_id: Query):
        pass

    @GET("/v1/walkthrough/force-like-illusts")
    def getForceLikeIllusts(self):
        pass

    @Authenticated
    @GET("/v1/illust/detail?filter=for_android")
    def getIllust(self, illust_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/bookmark-tags/illust")
    def getIllustBookmarkTags(self, user_id: Query, restrict: Query):
        pass

    @Authenticated
    @GET("/v1/user/browsing-history/illusts")
    def getIllustBrowsingHistory(self):
        pass

    @Authenticated
    @GET("/v2/illust/comments")
    def getIllustComments(self, illust_id: Query):
        pass

    @Authenticated
    @GET("/v1/illust/ranking?filter=for_android")
    def getIllustRanking(self, mode: Query, date: Query):
        pass

    @Authenticated
    @GET("/v2/illust/related?filter=for_android")
    def getIllustRecommended(self, illust_id: Query):
        pass

    @Authenticated
    @GET("/v1/illust/comment/replies")
    def getIllustReplyComments(self, comment_id: Query):
        pass

    @Authenticated
    @GET("/v1/illust-series/illust?filter=for_android")
    def getIllustSeriesByIllustId(self, illust_id: Query):
        pass

    @Authenticated
    @GET("/v1/illust/series?filter=for_android")
    def getIllustSeriesDetailsBySeriesId(self, illust_series_id: Query):
        pass

    @Authenticated
    @GET("/v1/trending-tags/illust?filter=for_android")
    def getIllustTrendTags(self):
        pass

    @Authenticated
    @GET("/v1/user/bookmarks/illust")
    def getLikeIllust(self, user_id: Query, restrict: Query, tag: Query):
        pass

    @Authenticated
    @GET("/v2/illust/bookmark/detail")
    def getLikeIllustDetail(self, illust_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/bookmarks/novel")
    def getLikeNovel(self, user_id: Query, restrict: Query, tag: Query):
        pass

    @Authenticated
    @GET("/v2/novel/bookmark/detail")
    def getLikeNovelDetail(self, novel_id: Query):
        pass

    @Authenticated
    @GET("/v1/illust/bookmark/users?filter=for_android")
    def getLikedIllustUser(self, illust_id: Query):
        pass

    @Authenticated
    @GET("/v1/novel/bookmark/users")
    def getLikedNovelUser(self, novel_id: Query):
        pass

    @Authenticated
    @GET("v1/live/list")
    def getLiveList(self, list_type: Query):
        pass

    @Authenticated
    @POST("v1/mail-authentication/send")
    def getMailAuthentication(self):
        pass

    @Authenticated
    @GET("/v1/mute/list")
    def getMutedList(self):
        pass

    @Authenticated
    @GET("/v2/illust/mypixiv")
    def getMyPixivIllusts(self):
        pass

    @Authenticated
    @GET("/v1/novel/mypixiv")
    def getMyPixivNovels(self):
        pass

    @Authenticated
    @GET("/v1/notification/new-from-following")
    def getNewFromFollowingNotification(self, latest_seen_illust_id: Query,
                                        latest_seen_novel_id: Query,
                                        last_notified_datetime: Query):
        pass

    @Authenticated
    @GET("/v1/illust/new?filter=for_android")
    def getNewIllust(self, content_type: Query):
        pass

    @Authenticated
    @GET("/v1/novel/new")
    def getNewNovel(self):
        pass

    @Authenticated
    @GET("/v1/notification/settings")
    def getNotificationSettings(self):
        pass

    @Authenticated
    @GET("/v2/novel/detail")
    def getNovel(self, novel_id: Query):
        pass

    @Authenticated
    @GET("v1/user/bookmark-tags/novel")
    def getNovelBookmarkTags(self, user_id: Query, restrict: Query):
        pass

    @Authenticated
    @GET("/v1/user/browsing-history/novels")
    def getNovelBrowsingHistory(self):
        pass

    @Authenticated
    @GET("/v2/novel/comments")
    def getNovelComments(self, novel_id: Query):
        pass

    @Authenticated
    @GET("v2/novel/markers")
    def getNovelMarkers(self):
        pass

    @Authenticated
    @GET("/v1/novel/ranking")
    def getNovelRanking(self, mode: Query, date: Query):
        pass

    @Authenticated
    @GET("/v1/novel/comment/replies")
    def getNovelReplyComments(self, comment_id: Query):
        pass

    @Authenticated
    @GET("/v1/novel/series")
    def getNovelSeries(self, series_id: Query):
        pass

    @Authenticated
    @GET("/v1/novel/text")
    def getNovelText(self, novel_id: Query):
        pass

    @Authenticated
    @GET("/v1/trending-tags/novel")
    def getNovelTrendTags(self):
        pass

    @Authenticated
    @GET("/v1/spotlight/articles?filter=for_android")
    def getPixivisionArticles(self, category: Query):
        pass

    @Authenticated
    @GET("/v1/search/popular-preview/illust?filter=for_android")
    def getPopularPreviewIllust(self, word: Query, search_target: Query,
                                start_date: Query, end_date: Query):
        pass

    @Authenticated
    @GET("/v1/search/popular-preview/novel")
    def getPopularPreviewNovel(self, word: Query, search_target: Query,
                               start_date: Query, end_date: Query):
        pass

    @GET("/v1/user/profile/presets")
    def getProfilePresets(self):
        pass

    @Authenticated
    @GET("/v1/illust/recommended?filter=for_android")
    def getRecommendedIllusts(self, include_ranking_illusts: Query):
        pass

    @Authenticated
    @GET("/v1/manga/recommended?filter=for_android")
    def getRecommendedMangaList(self, include_ranking_illusts: Query,
                                bookmark_illust_ids: Query):
        pass

    @Authenticated
    @GET("/v1/novel/recommended")
    def getRecommendedNovels(self, include_ranking_novels: Query):
        pass

    @Authenticated
    @GET("/v1/search/autocomplete")
    def getSearchAutoCompleteKeywords(self, word: Query):
        pass

    @Authenticated
    @GET("/v1/search/illust?filter=for_android")
    def getSearchIllust(self, word: Query, sort: Query, search_target: Query,
                        bookmark_num_min: Query, bookmark_num_max: Query,
                        start_date: Query, end_date: Query):
        pass

    @Authenticated
    @GET("/v1/search/bookmark-ranges/illust?filter=for_android")
    def getSearchIllustBookmarkRanges(self, word: Query, search_target: Query,
                                      start_date: Query, end_date: Query):
        pass

    @Authenticated
    @GET("/v1/search/novel")
    def getSearchNovel(self, word: Query, sort: Query, search_target: Query,
                       bookmark_num_min: Query, bookmark_num_max: Query,
                       start_date: Query, end_date: Query):
        pass

    @Authenticated
    @GET("/v1/search/bookmark-ranges/novel")
    def getSearchNovelBookmarkRanges(self, word: Query, search_target: Query,
                                     start_date: Query, end_date: Query):
        pass

    @Authenticated
    @GET("/v1/search/user?filter=for_android")
    def getSearchUser(self, word: Query):
        pass

    @Authenticated
    @GET("v1/ugoira/metadata")
    def getUgoiraMetadata(self, illust_id: Query):
        pass

    @Authenticated
    @POST("v1/upload/status")
    def getUploadIllustStatus(self, convert_key: Field):
        pass

    @Authenticated
    @GET("/v1/user/detail?filter=for_android")
    def getUser(self, user_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/follower?filter=for_android")
    def getUserFollower(self, user_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/following?filter=for_android")
    def getUserFollowing(self, user_id: Query, restrict: Query):
        pass

    @Authenticated
    @GET("/v1/user/illust-series")
    def getUserIllustSeries(self, user_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/illusts?filter=for_android")
    def getUserIllusts(self, user_id: Query, type: Query):
        pass

    @Authenticated
    @GET("/v1/user/me/state")
    def getUserMeState(self):
        pass

    @Authenticated
    @GET("/v1/user/mypixiv?filter=for_android")
    def getUserMyPixiv(self, user_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/novels")
    def getUserNovels(self, user_id: Query):
        pass

    @Authenticated
    @GET("/v1/user/recommended?filter=for_android")
    def getUserRecommended(self):
        pass

    @Authenticated
    @GET("/v1/user/related?filter=for_android")
    def getUserRelated(self, seed_user_id: Query):
        pass

    @GET("v1/walkthrough/illusts")
    def getWalkthroughIllusts(self):
        pass

    @Authenticated
    @POST("/v2/user/browsing-history/illust/add")
    def postAddIllustBrowsingHistory(self, illust_ids: Field):
        pass

    @Authenticated
    @POST("/v2/user/browsing-history/novel/add")
    def postAddNovelBrowsingHistory(self, novel_ids: Field):
        pass

    @Authenticated
    @POST("v1/novel/marker/add")
    def postAddNovelMarker(self, novel_id: Field, page: Field):
        pass

    @Authenticated
    @POST("/v1/dev/ios/receipt/post")
    def postDebugReceipt(self, receipt_data: Field):
        pass

    @Authenticated
    @POST("/v1/illust/delete")
    def postDeleteIllust(self, illust_id: Field):
        pass

    @Authenticated
    @POST("/v1/novel/delete")
    def postDeleteNovel(self, novel_id: Field):
        pass

    @Authenticated
    @POST("v1/novel/marker/delete")
    def postDeleteNovelMarker(self, novel_id: Field):
        pass

    @Authenticated
    @POST("/v1/notification/settings/edit")
    def postEditNotificationSettings(self, enabled_notification_types: Field):
        pass

    @Authenticated
    @POST("/v1/feedback")
    def postFeedback(self, message: Field, device: Field, dimension01: Field,
                     dimension02: Field, dimension03: Field, dimension04: Field):
        pass

    @Authenticated
    @POST("/v1/user/follow/add")
    def postFollowUser(self, user_id: Field, restrict: Field):
        pass

    @Authenticated
    @POST("/v1/illust/comment/add")
    def postIllustComment(self, illust_id: Field, comment: Field,
                          parent_comment_id: Field):
        pass

    @Authenticated
    @POST("/v2/illust/bookmark/add")
    def postLikeIllust(self, illust_id: Field, restrict: Field, tags: Field):
        pass

    @Authenticated
    @POST("/v2/novel/bookmark/add")
    def postLikeNovel(self, novel_id: Field, restrict: Field, tags: Field):
        pass

    @Authenticated
    @POST("/v1/mute/edit")
    def postMuteSetting(self, add_user_ids: Field, delete_user_ids: Field,
                        add_tags: Field, delete_tags: Field):
        pass

    @Authenticated
    @POST("v1/novel/comment/add")
    def postNovelComment(self, novel_id: Field, comment: Field,
                         parent_comment_id: Field):
        pass

    @Authenticated
    @POST("v1/notification/user/register")
    def postRegisterForNotifications(self, timezone_offset: Field,
                                     disable_notifications: Field):
        pass

    @Authenticated
    @POST("/v1/premium/android/register")
    def postRegisterPremium(self, purchase_data: Field, signature: Field,
                            app_version: Field):
        pass

    @Authenticated
    @POST("/v1/user/follow/delete")
    def postUnfollowUser(self, user_id: Field):
        pass

    @Authenticated
    @POST("/v1/illust/bookmark/delete")
    def postUnlikeIllust(self, illust_id: Field):
        pass

    @Authenticated
    @POST("/v1/novel/bookmark/delete")
    def postUnlikeNovel(self, novel_id: Field):
        pass

    # @Authenticated
    # @POST("v1/upload/illust")
    # def postUploadIllust(self, requestBody):
    #     pass

    # @Authenticated
    # @POST("/v1/user/profile/edit")
    # def postUserProfileEdit(self, requestBody):
    #     pass

    @Authenticated
    @POST("/v1/user/workspace/edit")
    def postUserWorkspaceEdit(self, pc: Field, monitor: Field, tool: Field,
                              scanner: Field, tablet: Field, mouse: Field,
                              printer: Field, desktop: Field, music: Field,
                              desk: Field, chair: Field, comment: Field):
        pass
